/* ------------------- Joystick Gamer USB v0.2 ---------------------
 *  
 * Pruebas para la implementación de un joystick XBox conectado a Xbox Adaptative Controller.
 * 
 * Version 0.2:  
 *   Se definen 2 entradas analógicas para controlar un joystick basado en potenciómetros:
 *     Eje X se conecta en A0
 *     Eje y se conecta en A1
 *   Se definen 4 entradas para definir 4 botones de disparo.
 *         
 *   Las 4 entradas digitales se definen en el array pinButtons siempre en este orden:
 *   [arriba, derecha, abajo, izquierda, Fire 1, Fire 2, Fire 3, Fire 4]
 * 
 * Version 0.1:  
 *   Se definen 4 entradas para controlar los 4 ejes de movimento
 *   Se definen 4 entradas para definir 4 botones de disparo.
 *         
 *   Las 8 entradas digitales se definen en el array pinButtons siempre en este orden:
 *   [arriba, derecha, abajo, izquierda, Fire 1, Fire 2, Fire 3, Fire 4]
 *   
 * Requisitos:
 *   Tener instalada la libreria Arduino Joystick versión 2.1.1
 *   https://github.com/MHeironimus/ArduinoJoystickLibrary
 *   
 * Author: Asociación MóstolesMakers
 *         -Instagram: @mostolesmakers 
 *       
 * ----------------------------------------------------------------------
 * Changelog:
 */

#include <Joystick.h>

Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID,JOYSTICK_TYPE_GAMEPAD,
  32, 0,                 // Numero de botones, Hat Switch Count
  true, true, false,     // Eje X (habilitado), Eje Y (habilitado), Eje Z  (deshabilitado)
  false, false, false,   // No Rx, Ry, or Rz
  false, false,          // No rudder or throttle
  false, false, false);  // No accelerator, brake, or steering

// Pines configurados
int pinButtons[4] = {16,10,14,15};

// Ultimo estado de las entradas
int lastButtonState[4] = {0,0,0,0};

int LEFT_THRESHOLD = 400;
int RIGHT_THRESHOLD= 800;
int UP_THRESHOLD   = 400;
int DOWN_THRESHOLD = 800;

int xValue = 0 ; // Eje X
int yValue = 0 ; // Eje Y

void setup() {
  // Configurar entradas digitales disparo
  for (int index = 0; index < 4; index++)
  {
    pinMode(pinButtons[index] , INPUT_PULLUP);
  }

  // Initialize Joystick Library
  Joystick.begin();
  Joystick.setXAxisRange(-1, 1);
  Joystick.setYAxisRange(-1, 1);
}


void loop() {

  // Leer entradas digitales para disparo
  for (int index = 0; index < 4; index++)
  {
    int currentButtonState = !digitalRead(pinButtons[index]);
    if (currentButtonState != lastButtonState[index])
    {
      switch (index) {
        case 0: // FIRE
          Joystick.setButton(4, currentButtonState);  // Pulsador "A" de la XBox
          break;
        case 1: // FIRE
          Joystick.setButton(5, currentButtonState);  // Pulsador "B" de la XBox
          break;
        case 2: // FIRE
          Joystick.setButton(6, currentButtonState);
          break;
        case 3: // FIRE
          Joystick.setButton(7, currentButtonState);
          break;
      }
      lastButtonState[index] = currentButtonState;
    }
  }

  // Leer entradas analogicas para joystick
  xValue = analogRead(A0);
  yValue = analogRead(A1);
  if (xValue < LEFT_THRESHOLD) {
    Joystick.setXAxis(-1);
  } else if (xValue > RIGHT_THRESHOLD) {
    Joystick.setXAxis(1);
  } else {
    Joystick.setXAxis(0);
  }

  // check up/down commands
  if (yValue < UP_THRESHOLD) {
    Joystick.setYAxis(1);
  } else if (yValue > DOWN_THRESHOLD) {
    Joystick.setYAxis(-1);
  } else {
    Joystick.setYAxis(0);
  }

  delay(10);
}
